// MySql_Monitor.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#define BUFFER_SIZE 1024

MYSQL *conn;
char *server;
char *user;
char *password; 
char *database;
char *path;


int main(int argc, char* argv[])
{
    size_t count;

    int i=1;
    while ((i< argc) && (argv[i][0] == '-'))
	{
		switch (argv[i][1])
		{
			case 'h':
				i++;
                server = argv[i];
				break;

			case 'u':
				i++;
				user = argv[i];
				break;

            case 'd':
				i++;
				database = argv[i];
				break;

			case 'p':
				password = &argv[i][2];
				break;
		}
        i++;
	}

	path = argv[i];

    conn = mysql_init(NULL);

   /* ** test connection ** */
   /* Connect to database */
   if (!mysql_real_connect(conn, server,
         user, password, database, 0, NULL, 0)) {
      fprintf(stderr, "%s\n", mysql_error(conn));
      exit(1);
   }

   /* close connection */
   mysql_close(conn);

   /* start main loop */
   WatchDirectory((LPTSTR)path);
}

void WatchDirectory(LPTSTR lpDir)
{
   DWORD dwWaitStatus; 
   HANDLE dwChangeHandles[1]; 
   TCHAR lpDrive[4];
   TCHAR lpFile[_MAX_FNAME];
   TCHAR lpExt[_MAX_EXT];

   _tsplitpath_s(lpDir, lpDrive, 4, NULL, 0, lpFile, _MAX_FNAME, lpExt, _MAX_EXT);

   lpDrive[2] = (TCHAR)'\\';
   lpDrive[3] = (TCHAR)'\0';
 
   /* Watch the directory for file creation and deletion. */
   dwChangeHandles[0] = FindFirstChangeNotification( 
      lpDir,                         /* directory to watch */ 
      FALSE,                         /* do not watch subtree  */
      FILE_NOTIFY_CHANGE_FILE_NAME | /* watch file name changes */
      FILE_NOTIFY_CHANGE_SIZE        /* watch file size changes */
      ); 
 
   if (dwChangeHandles[0] == INVALID_HANDLE_VALUE) 
   {
     printf("\n ERROR: FindFirstChangeNotification function failed.\n");
     ExitProcess(GetLastError()); 
   }
 

   /* Make a final validation check on our handles. */

   if (dwChangeHandles[0] == NULL)
   {
     printf("\n ERROR: Unexpected NULL from FindFirstChangeNotification.\n");
     ExitProcess(GetLastError()); 
   }

    /* Change notification is set. Now wait on both notification 
         handles and refresh accordingly. */
 
   while (TRUE) 
   { 
    /* Wait for notification. */
 
      printf("\nWaiting for notification...\n");

      dwWaitStatus = WaitForMultipleObjects(1, dwChangeHandles, FALSE, INFINITE); 
 
      switch (dwWaitStatus) 
      { 
         case WAIT_OBJECT_0: 
 
         /* A file was created, renamed, or deleted in the directory.
          Refresh this directory and restart the notification. */
 
             RefreshDirectory(lpDir); 
             if ( FindNextChangeNotification(dwChangeHandles[0]) == FALSE )
             {
               printf("\n ERROR: FindNextChangeNotification function failed.\n");
               ExitProcess(GetLastError()); 
             }
             break; 
 
         case WAIT_TIMEOUT:

         /* A timeout occurred, this would happen if some value other 
          than INFINITE is used in the Wait call and no changes occur.
          In a single-threaded environment you might not want an
          INFINITE wait. */
 
          /*  printf("\nNo changes in the timeout period.\n"); */
            break;

         default: 
            printf("\n ERROR: Unhandled dwWaitStatus.\n");
            ExitProcess(GetLastError());
            break;
      }
   }
}

void RefreshDirectory(LPTSTR lpDir)
{

   time_t t;
   struct tm *lt;
   
   conn = mysql_init(NULL);
   /* Connect to database */
   if (!mysql_real_connect(conn, server,
         user, password, database, 0, NULL, 0)) {
      fprintf(stderr, "%s\n", mysql_error(conn));
      exit(1);
   }
   

    t = time (NULL);
    lt = localtime (&t);


   _tprintf(TEXT("Directory (%s) changed at %04d-%02d-%02d %02d:%02d:%02d\n"), lpDir,lt->tm_year+1900, lt->tm_mon+1, lt->tm_mday,
           lt->tm_hour, lt->tm_min, lt->tm_sec);

   show_processlist();
   show_open_tables();
   show_innodb_status();


   /* close connection */
   mysql_close(conn);
}

void show_processlist(){
   MYSQL_RES *res;
   MYSQL_ROW row;
   
   /* send SQL query */
   if (mysql_query(conn, "show full processlist")) {
      fprintf(stderr, "%s\n", mysql_error(conn));
      exit(1);
   }
   res = mysql_use_result(conn);

   /* output table name */
   printf("MySQL full processlist:\n");
   while ((row = mysql_fetch_row(res)) != NULL)
      printf("%s %s %s %s %s %s - %s\n", row[0], row[1],row[2],row[3],row[4],row[5], row[7]);

   mysql_free_result(res);
}

void show_open_tables(){
   MYSQL_RES *res;
   MYSQL_ROW row;
   
   /* send SQL query */
   if (mysql_query(conn, "show open tables")) {
      fprintf(stderr, "%s\n", mysql_error(conn));
      exit(1);
   }
   res = mysql_use_result(conn);

   /* output table name */
   printf("MySQL Open Tables:\n");
   while ((row = mysql_fetch_row(res)) != NULL)
      printf("%s %s %s %s \n", row[0], row[1],row[2],row[3]);

   mysql_free_result(res);
}


void show_innodb_status(){
   MYSQL_RES *res;
   MYSQL_ROW row;
   
   /* send SQL query */
   if (mysql_query(conn, "SHOW ENGINE INNODB STATUS")) {
      fprintf(stderr, "%s\n", mysql_error(conn));
      exit(1);
   }
   res = mysql_use_result(conn);
   /* output table name */
   printf("MySQL InnoDB Status:\n");
   while ((row = mysql_fetch_row(res)) != NULL)
      printf("%s %s %s \n", row[0], row[1],row[2]);

   mysql_free_result(res);
}